/*
 * Copyright (c) 2008, Witold Wysota
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * The name of Witold Wysota may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <QtCore>
#include <QtGui>

class FindPrimes : public QObject {
  Q_OBJECT
public:
  FindPrimes(QObject *parent = 0) : QObject(parent){}
public slots:
  void start(qlonglong _max){
    emit prime(1); emit prime(2); emit prime(3);
    max = _max; cand = 5; curr = 3;
    next();
    QTimer::singleShot(0, this, SLOT(calculate())); 
  }
private slots:
  void calculate(){
    QTime t;
    t.start();
    while(t.elapsed()<50){
      if(cand>max){ emit finished(); return; } // end
      if(curr>sqrt){ emit prime(cand); next(); } //prime
      else if(cand % curr == 0) next(); // not prime
      else curr+=2; // check next divisor
    }
    QTimer::singleShot(0, this, SLOT(calculate()));
  }
signals:
  void prime(qlonglong);
  void finished();
private:
  qlonglong cand, max, curr;
  double sqrt;
  void next(){ cand+=2; curr = 3; sqrt = ::sqrt(cand);}
};

class Widget : public QWidget {
  Q_OBJECT
public:
  Widget() : QWidget(){
    QVBoxLayout *l = new QVBoxLayout(this);
    l->addWidget(new QLabel(tr("Calculate primes...")));
    QHBoxLayout *hl = new QHBoxLayout;
    sb = new QSpinBox;
    sb->setMinimum(3);
    sb->setMaximum(99999999);
    sb->setValue(100000);
    pb = new QPushButton("GO");
    listWidget = new QListWidget;
    hl->addWidget(sb);
    hl->addWidget(pb);
    l->addLayout(hl);
    l->addWidget(listWidget);
    connect(pb, SIGNAL(clicked()), SLOT(start()));
    connect(&primes, SIGNAL(finished()), SLOT(finished()));
    connect(&primes, SIGNAL(prime(qlonglong)), SLOT(newPrime(qlonglong)));
  }
public slots:
  void start(){
    listWidget->clear();
    pb->setEnabled(false);
    sb->setEnabled(false);
    primes.start(sb->value());
  }
private slots:
  void newPrime(qlonglong val){
    new QListWidgetItem(QString::number(val), listWidget);
  }
  void finished(){
    pb->setEnabled(true);
    sb->setEnabled(true);
  }
private:
  FindPrimes primes;
  QListWidget *listWidget;
  QSpinBox *sb;
  QPushButton *pb;
};

#include "main.moc"

int main(int argc, char **argv){
  QApplication app(argc, argv);
  Widget w;
  w.show();
  return app.exec();
}
